{{ certificates.cert_param }} {{ certificates.cert_file }} {{ certificates.key_param }} {{ certificates.key_file }} {{ protocol.additional_parameters }} {% if network.interface %} {{ network.interface.param }} {{ network.interface.value }} {% endif %} {{ network.destination.param}} {{ network.destination.value}}:{{ network.port }} > {{ logging.log_path }} 2> {{ logging.err_path }}